## begin license ##
#
#    BoushiStorage stores data in a reliable, extendable BerkeleyDB storage
#    with great performance.
#    Copyright (C) 2006-2008 Seek You Too B.V. (CQ2) http://www.cq2.nl
#    Copyright (C) 2010 Universiteit van Tilburg (UvT) http://www.uvt.nl
#
#    This file is part of BZV.
#
#    BZV is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    BZV is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with BZV; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##

# $Id: boushistorage.py 3959 2010-09-14 07:16:09Z place $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.4.1/src/boushistorage.py $

from boushi import Boushi
from cStringIO import StringIO
from re import compile, escape

# from meresco storage.py:
BAD_CHARS = map(chr, range(32)) + ['/', '%', ',']

FIND_BAD_CHARS = compile('(^\.|' + '|'.join(map(escape, BAD_CHARS)) + ')')
CHAR_TO_HEX = lambda x: '%%%02X' % ord(x.group(1))

REVERSE_BAD_CHARS = compile('%([0-9A-F]{2})')
HEX_TO_CHAR = lambda x: chr(int(x.group(1), 16))

def escapeName(name):
    return FIND_BAD_CHARS.sub(CHAR_TO_HEX, str(name))

def unescapeName(name):
    return REVERSE_BAD_CHARS.sub(HEX_TO_CHAR, str(name))

class BoushiStorage(object):
    def __init__(self, directory, split = lambda x:(x,), join=lambda x:''.join(x)):
        #print 'BOUSHI: '+directory+' opened'
        self._storage = Boushi(directory)
        self._split = split
        self._join = join

    def _name2path(self, name):
        return str('/'.join(map(escapeName, self._split(name))))

    def _path2name(self, path):
        return self._join(map(unescapeName, path.split('/')))

    def put(self, name):
        return Sink(self._storage, self._name2path(name))

    def get(self, name):
        #print 'BOUSHI: '+self._name2path(name)+' get'
        return StringIO(self._storage[self._name2path(name)])

    def delete(self, name):
        #print 'BOUSHI: '+self._name2path(name)+' del'
        del self._storage[self._name2path(name)]

    def deletePart(self, id, part):
        prefix, identifier = id.split(":", 1)
        tobedeleted = "/".join((prefix, escapeName(identifier), part))
        del self._storage[tobedeleted]

    def __contains__(self, (id, part)):
        path = self._name2path((id, part))
        if part is None:
            for part in self._storage.ls(path):
                return True
            return False
        #print 'BOUSHI: '+path+' has'
        return path in self._storage

    def __iter__(self):
        return self._storage.find('')

    def glob(self, (prefix, part)):
        for path in self._storage.find(self._name2path((prefix, None))):
            yield self._path2name(path)

class Sink(object):
    def __init__(self, boushi, path):
        self._boushi = boushi
        self._path = path
        self._value = StringIO()

    def send(self, buf):
        self._value.write(str(buf))

    def fileno(self):
        raise NotImplementedError('fileno() not implemented for BoushiStorage')

    def close(self):
        #print 'BOUSHI: '+self._path+' put'
        self._boushi[self._path] = self._value.getvalue()
        self._value.close()

class StorageError(Exception):
    pass
